package com.telpo.dipperposition.config;

import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.Executor;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @program: DataPushServer
 * @description:
 * @author: linwl
 * @create: 2020-07-11 09:17
 */
@Configuration
@Slf4j
public class AsyncExecutorConfig implements AsyncConfigurer {

  @Value("${async.pool.corePoolSize}")
  private int corePoolSize;

  @Value("${async.pool.maxPoolSize}")
  private int maxPoolSize;

  @Value("${async.pool.queueCapacity}")
  private int queueCapacity;

  @Bean(name = "asyncServiceExecutor")
  public Executor asyncServiceExecutor() {
    log.info("start asyncServiceExecutor");
    ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
    // 配置核心线程数
    executor.setCorePoolSize(corePoolSize);
    // 配置最大线程数
    executor.setMaxPoolSize(maxPoolSize);
    // 配置队列大小
    executor.setQueueCapacity(queueCapacity);
    // 配置线程池中的线程的名称前缀
    String threadNamePrefix = "async-pool-";
    executor.setThreadNamePrefix(threadNamePrefix);

    // 设置拒绝策略：当pool已经达到max size的时候，如何处理新任务
    // CALLER_RUNS：不在新线程中执行任务，而是有调用者所在的线程来执行
    executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
    // 执行初始化
    executor.initialize();
    return executor;
  }

  /**
   * 异步任务中异常处理
   *
   * @return
   */
  @Override
  public AsyncUncaughtExceptionHandler getAsyncUncaughtExceptionHandler() {
    return (arg0, arg1, arg2) -> {
      log.error("==========================" + arg0.getMessage() + "=======================", arg0);
      log.error("com.telpo.auth.exception method:" + arg1.getName());
    };
  }
}
